"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.register = void 0;
function register(context) {
    return {
        getCompletionsAtPosition,
        getDefinitionAtPosition,
        getDefinitionAndBoundSpan,
        getTypeDefinitionAtPosition,
        getImplementationAtPosition,
        findRenameLocations,
        getReferencesAtPosition,
        findReferences,
    };
    // apis
    function getCompletionsAtPosition(fileName, position, options) {
        const finalResult = context.getTsLs('script').getCompletionsAtPosition(fileName, position, options);
        if (finalResult) {
            finalResult.entries = finalResult.entries.filter(entry => entry.name.indexOf('__VLS_') === -1);
        }
        return finalResult;
    }
    function getReferencesAtPosition(fileName, position) {
        return findLocations(['script', 'template'], fileName, position, 'references');
    }
    function getDefinitionAtPosition(fileName, position) {
        return findLocations(['script'], fileName, position, 'definition');
    }
    function getTypeDefinitionAtPosition(fileName, position) {
        return findLocations(['script'], fileName, position, 'typeDefinition');
    }
    function getImplementationAtPosition(fileName, position) {
        return findLocations(['script', 'template'], fileName, position, 'implementation');
    }
    function findRenameLocations(fileName, position, findInStrings, findInComments, providePrefixAndSuffixTextForRename) {
        return findLocations(['script', 'template'], fileName, position, 'rename', findInStrings, findInComments, providePrefixAndSuffixTextForRename);
    }
    function findLocations(lsTypes, fileName, position, mode, findInStrings = false, findInComments = false, providePrefixAndSuffixTextForRename) {
        return lsTypes.map(lsType => worker(lsType)).flat();
        function worker(lsType) {
            const tsLs = context.getTsLs(lsType);
            const loopChecker = new Set();
            let symbols = [];
            if (tsLs)
                withTeleports(fileName, position, tsLs);
            return symbols.map(s => transformDocumentSpanLike(lsType, s)).filter(notEmpty);
            function withTeleports(fileName, position, tsLs) {
                if (loopChecker.has(fileName + ':' + position))
                    return;
                loopChecker.add(fileName + ':' + position);
                const _symbols = mode === 'definition' ? tsLs.getDefinitionAtPosition(fileName, position)
                    : mode === 'typeDefinition' ? tsLs.getTypeDefinitionAtPosition(fileName, position)
                        : mode === 'references' ? tsLs.getReferencesAtPosition(fileName, position)
                            : mode === 'implementation' ? tsLs.getImplementationAtPosition(fileName, position)
                                : mode === 'rename' ? tsLs.findRenameLocations(fileName, position, findInStrings, findInComments, providePrefixAndSuffixTextForRename)
                                    : undefined;
                if (!_symbols)
                    return;
                symbols = symbols.concat(_symbols);
                for (const ref of _symbols) {
                    loopChecker.add(ref.fileName + ':' + ref.textSpan.start);
                    const teleport = context.vueFiles.getTeleport(lsType, ref.fileName);
                    if (!teleport)
                        continue;
                    for (const [teleRange] of teleport.findTeleports(ref.textSpan.start, ref.textSpan.start + ref.textSpan.length, sideData => {
                        if ((mode === 'definition' || mode === 'typeDefinition' || mode === 'implementation') && !sideData.capabilities.definitions)
                            return false;
                        if ((mode === 'references') && !sideData.capabilities.references)
                            return false;
                        if ((mode === 'rename') && !sideData.capabilities.rename)
                            return false;
                        return true;
                    })) {
                        if (loopChecker.has(ref.fileName + ':' + teleRange.start))
                            continue;
                        withTeleports(ref.fileName, teleRange.start, tsLs);
                    }
                }
            }
        }
    }
    function getDefinitionAndBoundSpan(fileName, position) {
        return worker('script');
        function worker(lsType) {
            const tsLs = context.getTsLs(lsType);
            const loopChecker = new Set();
            let textSpan;
            let symbols = [];
            if (tsLs)
                withTeleports(fileName, position, tsLs);
            if (!textSpan)
                return;
            return {
                textSpan: textSpan,
                definitions: symbols?.map(s => transformDocumentSpanLike(lsType, s)).filter(notEmpty),
            };
            function withTeleports(fileName, position, tsLs) {
                if (loopChecker.has(fileName + ':' + position))
                    return;
                loopChecker.add(fileName + ':' + position);
                const _symbols = tsLs.getDefinitionAndBoundSpan(fileName, position);
                if (!_symbols)
                    return;
                if (!textSpan) {
                    textSpan = _symbols.textSpan;
                }
                if (!_symbols.definitions)
                    return;
                symbols = symbols.concat(_symbols.definitions);
                for (const ref of _symbols.definitions) {
                    loopChecker.add(ref.fileName + ':' + ref.textSpan.start);
                    const teleport = context.vueFiles.getTeleport(lsType, ref.fileName);
                    if (!teleport)
                        continue;
                    for (const [teleRange] of teleport.findTeleports(ref.textSpan.start, ref.textSpan.start + ref.textSpan.length, sideData => !!sideData.capabilities.definitions)) {
                        if (loopChecker.has(ref.fileName + ':' + teleRange.start))
                            continue;
                        withTeleports(ref.fileName, teleRange.start, tsLs);
                    }
                }
            }
        }
    }
    function findReferences(fileName, position) {
        const scriptResult = worker('script');
        const templateResult = worker('template');
        return [
            ...scriptResult,
            ...templateResult,
        ];
        function worker(lsType) {
            const tsLs = context.getTsLs(lsType);
            const loopChecker = new Set();
            let symbols = [];
            if (tsLs)
                withTeleports(fileName, position, tsLs);
            return symbols.map(s => transformReferencedSymbol(lsType, s)).filter(notEmpty);
            function withTeleports(fileName, position, tsLs) {
                if (loopChecker.has(fileName + ':' + position))
                    return;
                loopChecker.add(fileName + ':' + position);
                const _symbols = tsLs.findReferences(fileName, position);
                if (!_symbols)
                    return;
                symbols = symbols.concat(_symbols);
                for (const symbol of _symbols) {
                    for (const ref of symbol.references) {
                        loopChecker.add(ref.fileName + ':' + ref.textSpan.start);
                        const teleport = context.vueFiles.getTeleport(lsType, ref.fileName);
                        if (!teleport)
                            continue;
                        for (const [teleRange] of teleport.findTeleports(ref.textSpan.start, ref.textSpan.start + ref.textSpan.length, sideData => !!sideData.capabilities.references)) {
                            if (loopChecker.has(ref.fileName + ':' + teleRange.start))
                                continue;
                            withTeleports(ref.fileName, teleRange.start, tsLs);
                        }
                    }
                }
            }
        }
    }
    // transforms
    function transformReferencedSymbol(lsType, symbol) {
        const definition = transformDocumentSpanLike(lsType, symbol.definition);
        const references = symbol.references.map(r => transformDocumentSpanLike(lsType, r)).filter(notEmpty);
        if (definition) {
            return {
                definition,
                references,
            };
        }
        else if (references.length) { // TODO: remove patching
            return {
                definition: {
                    ...symbol.definition,
                    fileName: references[0].fileName,
                    textSpan: references[0].textSpan,
                },
                references,
            };
        }
    }
    function transformDocumentSpanLike(lsType, documentSpan) {
        const textSpan = transformSpan(lsType, documentSpan.fileName, documentSpan.textSpan);
        if (!textSpan)
            return;
        const contextSpan = transformSpan(lsType, documentSpan.fileName, documentSpan.contextSpan);
        const originalTextSpan = transformSpan(lsType, documentSpan.originalFileName, documentSpan.originalTextSpan);
        const originalContextSpan = transformSpan(lsType, documentSpan.originalFileName, documentSpan.originalContextSpan);
        return {
            ...documentSpan,
            fileName: textSpan.fileName,
            textSpan: textSpan.textSpan,
            contextSpan: contextSpan?.textSpan,
            originalFileName: originalTextSpan?.fileName,
            originalTextSpan: originalTextSpan?.textSpan,
            originalContextSpan: originalContextSpan?.textSpan,
        };
    }
    function transformSpan(lsType, fileName, textSpan) {
        if (!fileName)
            return;
        if (!textSpan)
            return;
        for (const vueLoc of context.vueFiles.fromEmbeddedLocation(lsType, fileName, textSpan.start, textSpan.start + textSpan.length)) {
            return {
                fileName: vueLoc.fileName,
                textSpan: {
                    start: vueLoc.range.start,
                    length: vueLoc.range.end - vueLoc.range.start,
                },
            };
        }
    }
}
exports.register = register;
function notEmpty(value) {
    return value !== null && value !== undefined;
}
//# sourceMappingURL=apis.js.map